
#include <QProcess>
#include <QGraphicsScene>
#include <QGraphicsLinearLayout>
#include <QMessageBox>
#include <QDBusMessage>
#include <QDBusConnection>
#include <QFileInfo>

#include <KStandardDirs>
#include <KLocale>
#include <KToolInvocation>
#include <KFilePlacesModel>
#include <KFileItem>
#include <KRun>
#include <konq_operations.h>

#include <Plasma/ToolTipManager>

#include "places.h"

using namespace Plasma;

Places::Places(QObject *parent, const QVariantList &args)
    : PopupApplet(parent, args)
{
    setBackgroundHints(DefaultBackground);
    setAspectRatioMode(IgnoreAspectRatio);
    setHasConfigurationInterface(false);
    KGlobal::locale()->insertCatalog("libkonq");
    KGlobal::locale()->insertCatalog("dolphin");
    m_open_exe = KStandardDirs::findExe("kioclient");
}


Places::~Places()
{
}

void Places::init()
{
    KFilePlacesModel *placemodel = new KFilePlacesModel;

    m_place = new KFilePlacesView;
    m_place->setAttribute(Qt::WA_NoSystemBackground);
    m_place->setDropOnPlaceEnabled(true);
    m_place->setModel(placemodel);
    m_place->clearSelection();

    m_pop = new QGraphicsProxyWidget(this);
    m_pop->setMinimumSize(300, (placemodel->rowCount() - placemodel->hiddenCount() + 3)*42);

    QGraphicsLinearLayout *vl = new QGraphicsLinearLayout(Qt::Vertical, m_pop);
    vl->addItem(scene()->addWidget(m_place));

    setGraphicsWidget(m_pop);
    setPopupIcon("user-home");

    Plasma::ToolTipManager::self()->setContent(this, Plasma::ToolTipContent(i18nc("@title:window", "Places"), "", KIcon("user-home")));

    connect(m_place, SIGNAL(urlChanged(const KUrl&)), this, SLOT(openUrl(const KUrl&)));
    connect(m_place, SIGNAL(urlsDropped(const KUrl&, QDropEvent*, QWidget*)),
            this, SLOT(catchUrl(const KUrl&, QDropEvent*, QWidget*)));
}

void Places::openUrl(const KUrl &url)
{
    if( url.protocol() == "file" && QFileInfo(url.toLocalFile()).isDir() )
    {
	KRun::runUrl(url.pathOrUrl(), "inode/directory", m_place);
	return;
    }

    if( url.protocol() == "http" || url.protocol() == "https" )
    {
	KToolInvocation::invokeBrowser(url.pathOrUrl());
	return;
    }

    if( !m_open_exe.isEmpty() )
	QProcess::startDetached(m_open_exe, QStringList() << "exec" << url.pathOrUrl());
}

void Places::dragEnterEvent(QGraphicsSceneDragDropEvent*)
{
    m_place->clearSelection();
    showPopup();
}

void Places::catchUrl(const KUrl& destination, QDropEvent* event, QWidget* widget)
{
    const QMimeData* mimeData = event->mimeData();
    if (mimeData->hasFormat("application/x-kde-dndextract")) {
        QString remoteDBusClient = mimeData->data("application/x-kde-dndextract");
        QDBusMessage message = QDBusMessage::createMethodCall(remoteDBusClient, "/DndExtract",
                                                              "org.kde.DndExtract", "extractSelectedFilesTo");
        message.setArguments(QVariantList() << destination.path());
        QDBusConnection::sessionBus().call(message);
    } else {
        const KUrl::List urls = KUrl::List::fromMimeData(event->mimeData());
        const int urlsCount = urls.count();
        if (urlsCount == 0) {
            // TODO: handle dropping of other data
        } else if ((urlsCount == 1) && (urls.first() == destination)) {
            QMessageBox::critical(0, "Error", i18nc("@info:status", "A folder cannot be dropped into itself"));
        } else {
            KonqOperations::doDrop(KFileItem(), destination, event, widget);
        }
    }
}

#include "places.moc"
